        // Illustrate animation of a filled arc drawn full circle.
        // This approach tries to limit flicker by taking advantage
        // of the fact that the arc only needs to be cleared when it
        // returns to its starting position.
import java.awt.*;

public class AWTAnimate2 {
    public static void main(String[] args){
        AnimationFrame frame = new AnimationFrame();
        frame.addWindowListener(new WindowCloser());
        frame.setVisible(true);
        // Hand over to the frame to run forever.
        frame.run();
    }
}

    // Repeatedly request a canvas to paint itself.
class AnimationFrame extends Frame {
    public AnimationFrame(){
        // Set the frame's size.
        final int width = 200, height = 200;
        setSize(width,height);

        add(getCanvas());
    }

    public void run(){
        // Animate the drawing 50 times per second.
        final int sleepTime = 1000/50;
        AnimatedArcCanvas canvas = getCanvas();
        while(true){
            try{
                Thread.sleep(sleepTime);
                // Perform the next animation step.
                canvas.step();
            }
            catch(InterruptedException e){
            }
        }
    }

    protected AnimatedArcCanvas getCanvas(){
        return animationCanvas;
    }

    private AnimatedArcCanvas animationCanvas = new AnimatedArcCanvas();
}

    // Animate an arc through a full circle before starting again.
class AnimatedArcCanvas extends Canvas {
    public void paint(Graphics g){
        Dimension size = getSize();

        final int x = size.width/4, y = size.height/4;
        final int width = size.width/2, height = size.height/2;

        int sweep = getAngle();
        if(sweep == 0){
            // Clear the area of the arc.
            g.clearRect(x,y,width,height);
        }
        g.fillArc(x,y,width,height,startAngle,sweep);
    }

        // Perform the next animation step.
    public void step(){
        // Update the angle.
        setAngle(getAngle()+angleIncrement);

        // Draw the current arc with this component's graphics context.
        paint(getGraphics());
    }

    protected int getAngle(){
        return angle;
    }

    protected void setAngle(int a){
        if(Math.abs(a) <= circleDegrees){
            angle = a;
        }
        else{
            // Time to restart.
            angle = 0;
        }
    }

    // Always start at the 3 o'clock position.
    private final int startAngle = 0;
    // The number of degrees in a circle.
    private final int circleDegrees = 360;
    // Update the angle by a fixed amount.
    private final int angleIncrement = 12;
    // How many degrees of sweep for the arc.
    private int angle = 0;
}
