        // An up-market implementation of the VCR interface.
        // Basic operations plus variable speed FF and REW.
        // FF and REW operate even if the tape is not STOPPED.
class UpMarketVCR implements VCR {
    public boolean play(){
        if(getStatus() != NOTAPE){
            setStatus(PLAYING);
            getGui().start(PLAYING_SPEED);
            return true;
        }
        else{
            return false;
        }
    }
    
    public boolean pause(){
        if(getStatus() != NOTAPE){
            setStatus(PAUSED);
            // The GUI does not visually distinguish between stop and pause.
            getGui().stop();
            return true;
        }
        else{
            return false;
        }
    }
    
    public boolean stop(){
        if(getStatus() != NOTAPE){
            setStatus(STOPPED);
            getGui().stop();
            return true;
        }
        else{
            return false;
        }
    }
        // Increase the forward speed.
    public boolean ff(){
        if(getStatus() != NOTAPE){
            // If we are already moving forward, simply increase speed,
            // otherwise set the basic FAST_SPEED.
            if(getStatus() == FORWARD){
                setSpeed(getSpeed()+1);
            }
            else{
                setStatus(FORWARD);
                setSpeed(FAST_SPEED);
            }
            getGui().start(getSpeed());
            return true;
        }
        else{
            return false;
        }
    }

        // Increase the rewind speed.
    public boolean rew(){
        if(getStatus() != NOTAPE){
            // If we are already moving  backward, simply increase speed,
            // otherwise set the basic FAST_SPEED.
            if(getStatus() == BACKWARD){
                setSpeed(getSpeed()- 1);
            }
            else{
                setStatus(BACKWARD);
                setSpeed(-FAST_SPEED);
            }
            getGui().start(getSpeed());
            return true;
        }
        else{
            return false;
        }
    }
    
        // Load the requested tape.
        // Return false if a tape is already loaded.
    public boolean loadTape(String tapeName){
        if(getStatus() == NOTAPE){
            setStatus(STOPPED);
            getGui().changeTape(tapeName);
            return true;
        }
        else{
            return false;
        }
    }

    public boolean eject(){
        if(getStatus() != NOTAPE){
            setStatus(NOTAPE);
            getGui().changeTape("");
            return true;
        }
        else{
            return false;
        }
    }

    public int getStatus(){
        return status;
    }

    private void setStatus(int s){
        status = s;
    }

    private int getSpeed(){
        return speed;
    }

        // Allow the speed to be set externally.
    public void setSpeed(int s){
        if(Math.abs(s) <= MAX_SPEED){
            speed = s;
        }
    }

    private TapeDemo getGui(){
        return gui;
    }

    // This object's current status attribute.
    private int status = NOTAPE;
    // The normal playing speed, initial fast speed and maximum speed for ff/rew.
    private static final int PLAYING_SPEED = 1, FAST_SPEED = 5, MAX_SPEED = 15;
    private int speed = PLAYING_SPEED;
    // A GUI to demonstrate this implementation.
    private TapeDemo gui = new TapeDemo("Up Market VCR",this);
    { gui.setLocation(40,40);
    }
}
