import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

    // Illustrate the registerKeyboardAction methods of
    // The JComponent classes.
    // The color of a central panel may be changed via different
    // key stroke combinations attached to three buttons.
class SwingKeyStrokeFrame extends JFrame {
    public SwingKeyStrokeFrame() {
        final int width = 300, height = 100;
        setSize(width,height);

        Container contents = getContentPane();
        final JLabel promptLabel = new JLabel("Change the Color",JLabel.CENTER);
        final JPanel colorPanel = new JPanel();
        // Set up the buttons to change the color of the panel.
        final JPanel buttonPanel = setupButtons(colorPanel);

        // Cause the program to exit if the Q key is typed
        // over the application.
        colorPanel.registerKeyboardAction(
            new ActionListener(){
                public void actionPerformed(ActionEvent e){
                    // Print the associated command message.
                    System.out.println(e.getActionCommand());
                    System.exit(0);
                }
            },
            "Goodbye",
            KeyStroke.getKeyStroke('Q'),
            JComponent.WHEN_IN_FOCUSED_WINDOW
        );

        contents.add(promptLabel,"North");
        contents.add(colorPanel,"Center");
        contents.add(buttonPanel,"South");
    }

        // Setup three buttons to be activated on different
        // keystroke combinations. Each should set the panel
        // to a different background color.
    protected JPanel setupButtons(final JPanel colorPanel){
        final JButton red = new JButton("Red"),
                      green = new JButton("Green"),
                      blue = new JButton("Blue");
        final JPanel panel = new JPanel();

        // Define the common background setting action in a named
        // local class.
        class BackgroundSetter implements ActionListener {
            BackgroundSetter(Color color){
                this.color = color;
            }

            public void actionPerformed(ActionEvent e){
                colorPanel.setBackground(color);
                colorPanel.repaint();
            }

            private final Color color;
        }

        // Red when SHIFT-R is released.
        red.registerKeyboardAction(
            new BackgroundSetter(Color.red),
            KeyStroke.getKeyStroke(KeyEvent.VK_R,
                                   InputEvent.SHIFT_MASK,true),
            JComponent.WHEN_IN_FOCUSED_WINDOW 
        );
        red.setEnabled(true);

        // Green when CTRL-G is pressed and the button has the focus.
        green.registerKeyboardAction(
            new BackgroundSetter(Color.green),
            KeyStroke.getKeyStroke(KeyEvent.VK_G,InputEvent.CTRL_MASK),
            JComponent.WHEN_FOCUSED
        );

        // Blue when 'b' is pressed.
        blue.registerKeyboardAction(
            new BackgroundSetter(Color.blue),
            KeyStroke.getKeyStroke('b'),
            JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT
        );

        panel.add(red);
        panel.add(green);
        panel.add(blue);

        return panel;
    }
}
